#!usr/bin/env python3
"""
rules.py

Early implementation for criteria for the linter to check the project against.

See https://github.com/OpenNewsLabs/open-project-linter/issues/21 .
"""

import os

import git
import pygments.lexers as lexers

# Pygments lexer names that will parse files that are not code
NOT_CODE = ['markdown','BBCode', 'Groff', 'MoinMoin/Trac Wiki markup',
            'reStructuredText', 'TeX','Gettext Catalog', 'HTTP', 'IRC logs',
            'Todotxt']

def check_file_presence(keyword, directory):
    """
    Checks whether a given directory contains a file whose name contains
    a given string.

    Parameters
    ----------
    keyword : a string containing the term to search for
    directory: a string containing the path to the directory to search in

    Returns
    -------
    True if the keyword is found in the filenames present in
        the directory and the corresponding file has content
    None if the keyword is found in the filenames present in the
        directory but the file has size 0 B
    False if the keyword is not found in the filenames
        present in the directory
    """
    files = os.listdir(directory)
    # this is unfortunately nested, FIXME?
    for f in files:
        if os.path.isfile(f):
            # in case there's anything in the path that matches keyword,
            # only match the filename
            filename = os.path.split(f)[1]
            if keyword in filename:
                if check_for_file_content(os.path.join(directory,f)):
                    return True
                else:
                    return None

    # if loop finishes, file name not found
    return False


def check_for_file_content(filename):
    """Return True if the file has > 0 B, False otherwise."""
    # note this gives FileNotFoundError if there is no file at filepath
    return os.path.getsize(filename) > 0


def check_for_code(directory):
    """Return True if a code file is present (not just structured text)."""
    code_present = False
    for root, dirs, files in os.walk(directory):
        for f in files:
            code_present = guess_code_present(os.path.join(root, f))
            if code_present:
                return True
    return False

def guess_code_present(filepath):
    text = get_file_text(filepath)
    filename = os.path.split(filepath)[1]
    try:
        lexer = lexers.guess_lexer_for_filename(filename, text)
        if lexer.name not in NOT_CODE:
            return True
        else:
            return False
    except lexers.ClassNotFound:
        return False


def get_file_text(filepath):
    try:
        with open(filepath, 'r') as f:
            return f.read()
    except UnicodeDecodeError:
        return None


def detect_version_control(directory):
    """Figure out whether the directory is under version control."""
    # possibly in the future could recurse and see if it's a subfolder?
    version_control_system = None

    if os.path.isdir(os.path.join(directory, '.git/')):
        version_control_system = 'git'

    # Could add checks for others here: CVS, svn, hg

    return version_control_system


def check_multiple_branches(repository):
    repo = git.Repo(repository)
    branches = repo.branches
    if len(branches) > 1:
        return True
    else:
        return False


def check_for_develop_branch(repository, dev_branch_name):
    repo = git.Repo(repository)
    branches = repo.branches
    for branch in branches:
        if dev_branch_name in branch.name:
            break
    else:
        return False

    return True


def check_for_multiple_commits(repository):
    """ Return True if any branch has more than 2 commits ( in git log)"""
    repo = git.Repo(repository)
    branches = repo.branches
    multiple_commits = False
    for branch in branches:
        commit_log = [x for x in branch.log() if 'commit' in x.message]
        if len(commit_log) > 1:
            multiple_commits = True
    return multiple_commits


# TODO: figure out how to do this, flesh this out
def check_for_signed_commits(repository):
    repo = git.Repo(repository)
    branches = repo.branches
    for branch in branches:
        pass



#!/usr/bin/env python3

import openlinter

def main():
    print('invoking command_line.main()')

    # Set up parser and CLI
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group()
#    group.add_argument('-u', '--url',
#        help='The URL to the GitHub repository to check. Defaults to None.')
    group.add_argument('-d', '--directory', help="The local path to your repository's base directory. Defaults to the current working directory.",
       default=os.getcwd()
    )
    parser.add_argument('-r', '--rules', help='The path to the rules configuration file, a YAML file containing the rules you would like to check for. Defaults to current-working-directory/openlinter/rules.yml.',
        default=os.path.join(os.getcwd(), 'openlinter/rules.yml')
    )
    parser.add_argument('-v', '--version', action='version', version='0.1dev')
    args = parser.parse_args()

    openlinter.main(args.directory, args.rules)



"""
A setuptools based setup module for the Open Project Linter.

Based on https://github.com/pypa/sampleproject/blob/master/setup.py
"""

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README
with(open(path.join(here, 'README.md'), encoding='utf-8')) as f:
    long_description = f.read()

setup(
    name='open-project-linter',
    version='0.1dev',
    packages=find_packages(),
    install_requires=['gitpython', 'pyyaml'],
    license='Apache v2.0',
    description='Automatic checklist for open source project best practices.',
    long_description=long_description,
    url='https://github.com/OpenNewsLabs/open-project-linter',
    author='Frances Hocutt',
    author_email='frances.hocutt+pypi@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Documentation',
        'Topic :: Software Development :: Version Control',
    ],
    keywords='project linter setup documentation development',
    entry_points={
        'console_scripts': ['openlinter=openlinter.openlinter:main']
    }
)



#!usr/bin/env python3
"""
openlinter.py

Early implementation for an open source project linter.

See https://github.com/OpenNewsLabs/open-project-linter/issues/21 .
"""

from __future__ import absolute_import

import argparse
import os
import yaml

import openlinter.rules as rules

# FIXME: this is all not very functional or testable.
def main():
    # Set up parser and CLI
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group()
#    group.add_argument('-u', '--url',
#        help='The URL to the GitHub repository to check. Defaults to None.')
    group.add_argument('-d', '--directory', help="The local path to your repository's base directory. Defaults to the current working directory.",
       default=os.getcwd()
    )
    parser.add_argument('-r', '--rules', help='The path to the rules configuration file, a YAML file containing the rules you would like to check for. Defaults to current-working-directory/openlinter/rules.yml.',
        default=os.path.join(os.getcwd(), 'openlinter/rules.yml')
    )
    parser.add_argument('-v', '--version', action='version', version='0.1dev')
    args = parser.parse_args()

    # Read in rules
    with open(args.rules, 'r') as f:
        text = f.read()

    rule_set = yaml.safe_load(text)

    # Check directory/repository against rules
    if 'files_exist' in rule_set:
        """
        # TODO: Consider architecture: how to handle the result (print to stdout
        #       as we go, or pass around a result object?), how to handle interface
        #       strings (hard-coded or able to change/localize easily).
        """
        # FIXME: also unfortunately nested
        for files_to_check in rule_set['files_exist']:
            for f in files_to_check:
                for name in files_to_check[f]:
                    result = rules.check_file_presence(name, args.directory)
                    # If one exists with content, great, stop checking
                    if result:
                        output = '  {} exists and has content'.format(name)
                        print(output)
                        break
                    # Otherwise note that none of the names exist?
                    # TODO: could wrangle this a bit
                    # brainwane/sumanah: should this say "no $name type file found"
                    #                    or list the individual ones not present?
                    elif result is None:
                        output = '! {} exists but is empty'.format(name,
                            args.directory)
                    else:
                        output = '! {} not found in {}'.format(name, args.directory)
                    print(output)

    if 'code_exists' in rule_set:
        code_exists = rules.check_for_code(args.directory)
        if code_exists:
            output = '  code files detected'
        else:
            output = '! no code files found'
        print(output)

    if 'version_control' in rule_set:
        vcs = rules.detect_version_control(args.directory)
        if vcs:
            output = '  version control using {}'.format(vcs)
        else:
            output = '! version control system not detected'
        print(output)

    # Conditionals may need work here, TODO
    if 'detect_git_branches' in rule_set['version_control'] and vcs == 'git':
        branches = rules.check_multiple_branches(args.directory)
        if branches:
            output = '  multiple git branches found'
        else:
            output = '! fewer than 2 git branches found'
        print(output)

        # check for a dev/feature branch
        # TODO: pull some of this out into a function
        develop = False
        for name in rule_set['dev_branch_names']:
            if rules.check_for_develop_branch(args.directory, name):
                develop = True
                output = '  development branch "{}" found'.format(name)
                print(output)
        if not develop:
            output = '! no development branch found'
            print(output)

    # Conditionals may need work here, TODO
    if 'multiple_git_commits' in rule_set['version_control'] and vcs == 'git':
        multiple_commits = rules.check_for_multiple_commits(args.directory)
        if multiple_commits:
            output = '  multiple commits on a branch found'
        else:
            output = '! one or fewer commits on each branch'
        print(output)

    elif 'detect_git_branches' in rule_set['version_control']:
        print('! no git repository detected, could not check for git branches')

    else:
        pass

    # TODO: can add future checks, also needs refactoring


if __name__ == '__main__':
    main()
